#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 01/20/2000
#################################################################################
#
# Configure Puppeteer - Script part of Webchat designed to configure Puppeteers
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use String::Random;						# Random String generator

use Webchat::Database;						# Database handler
use Webchat::Dbm; 						# DBM Addition
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters
	Webchat::Dbm::DBMOpenConfig(\%config);			# Attach hash

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Authenthication
	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Print title

		# Initial screen	
		unless ( defined($param{'OLDPASS'}) || defined($param{'EMAIL'}))
		{ InitialDisplay($Database, $cgi, \%param); }

		# Information handling
		else
		{ InformationHandler($Database, $cgi, \%param); }
	}


	Webchat::Database::DocumentGetFooter($Database);	# HTML footer

	Webchat::Dbm::DBMClose(\%config);			# Release hash
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData							# Check value, $choice, Database, User
{
	#####################
	# Data members
	my $value = shift;					# Value to check 
	my $choice = shift;					# Choice of the matter

	my $Database = shift;					# Possible database handle
	my $Statement;						# SQL Statement

	my $user = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		$user = Webchat::Login::Unmangle($user);	# Unmange username
		
		# Prepare and execute
		$Statement = $Database->prepare("SELECT PuppeteerPassword FROM Puppeteer 
						WHERE PuppeteerLogin=?");
		$Statement->execute($user);

		# Retreive and finish
		my $crypt = $Statement->fetchrow();
		$Statement->finish();

		# Verify existence of user and validity of password
		if ( ( defined($crypt) ) && ( $crypt eq crypt($value, $crypt) ) )
		{ return 1; }					# Return true
	}

	# Lenght
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 5 characters
		if (($value =~ tr/a-zA-Z0-9@.\///) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Check for @ and . lenght
		if ( ($value =~ /\@/) && ($value =~ /\./) && (($value =~ tr/a-zA-Z0-9@.\///) > 5) )
		{ return 1; }					# Return true
	}

	return 0; 						# False if all else fails
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 1); }


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	
	my $choice = 0;						# Choice of action

	my %config;						# Configuration hash


	#####################
	# Program area	
	
	# Link DBM with hash
	Webchat::Dbm::DBMOpenConfig(\%config);

	# Display of information
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Primary security check
	# Password Validity
	if ( CheckDataPassword($Database, $param->{'USER'}, $param->{'OLDPASS'}) )
	{ $choice++; }						# Increment by one
	else
	{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config{'ErrPassword'}</CENTER>")); }


	# Password change
	if ( defined($param->{'ONEPASS'}) )
	{
		# Valid new passwords
		if ( (CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{ $choice++; }					# Inrement by one
		else
		{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config{'ErrMismatch'}</CENTER>")); }
	}	

	# E-Mail change
	elsif ( defined($param->{'EMAIL'}) )
	{
		# Valid new E-Mail
		if ( CheckDataEMail($param->{'EMAIL'}) )
		{ $choice++; }					# Inrement by one
		else
		{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config{'ErrEMail'}</CENTER>")); }
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Unmangle string
		$param->{'USER'} = Webchat::Login::Unmangle($param->{'USER'}); 

		# Password change
		if ( defined($param->{'ONEPASS'}) )
		{
			# Create random string generator
			my $rand = new String::Random;
			
			# Insert new password
			$Database->do("UPDATE Puppeteer SET PuppeteerPassword=? WHERE PuppeteerLogin=?",
					{}, crypt($param->{'ONEPASS'}, $rand->randpattern("ssssssss")),
					$param->{'USER'} );
		}
		
		# EMail change
		elsif ( defined($param->{'EMAIL'}) )
		{
			# Insert new password
			$Database->do("UPDATE Puppeteer SET PuppeteerEMail=? WHERE PuppeteerLogin=?",
					{}, $param->{'EMAIL'}, $param->{'USER'});
		}

		# Congratulations message
		Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config{'TagConfSuccess'}</CENTER>"));
	}

	# Table bottom
	Webchat::Table::MakeBottom($cgi);

	# Free DBMs from hashes
	Webchat::Dbm::DBMClose(\%config);
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	
	my %login;						# Login hash
	my %config;						# Config hash
	

	#####################
	# Program area	
	
	# Link DBMs with hashes
	Webchat::Dbm::DBMOpenLogin(\%login);
	Webchat::Dbm::DBMOpenConfig(\%config);

	# Password change
	# Display of information
	print $cgi->start_form();

	# Top HTML
	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config{'TagConfPassword'});
	Webchat::Table::MakeBlank($cgi);

	# Password section
	Webchat::Table::MakeValid($cgi, $login{'TagPasswordOld'}, $cgi->password_field('OLDPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('ONEPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagPasswordRepeat'}, $cgi->password_field('TWOPASS', '', 10, 8));
	Webchat::Table::MakeBlank($cgi);
	
	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, $param);
	print $cgi->end_form(), "\n";

	# Spacing
	print "<BR>\n<BR>\n";

	# Email change
	# Display of information
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config{'TagConfEMail'});
	Webchat::Table::MakeBlank($cgi);

	# Email section
	Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('OLDPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45));
	Webchat::Table::MakeBlank($cgi);

	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, $param);
	print $cgi->end_form(), "\n";

	# Free DBM and hash
	Webchat::Dbm::DBMClose(\%login);
	Webchat::Dbm::DBMClose(\%config);
}
